/* Copyright (c) 2011  Eric Milles <eric.milles@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef OutputComponent_h
#define OutputComponent_h


#include "OutputFormat.h"
#include "Platform.h"
#include "Player.h"
#include "Uncopyable.h"
#include <cstddef>
#include <ctime>
#include <functional>


class RSOUTPUT_API OutputComponent
:
	private Uncopyable
{
public:
	explicit OutputComponent(Player&);
	~OutputComponent();

	void open(const OutputFormat&);
	void close();
	void reset();

	std::time_t latency() const;
	std::size_t buffered() const;
	std::size_t canWrite() const;

	bool write(const void*, std::size_t);

	bool setPaused(bool);
	void setBalance(double);
	void setVolume(double);

	typedef std::tr1::function<void (std::size_t)> ProgressCallback;
	void setProgressCallback(ProgressCallback);

private:
	class OutputComponentImpl* const _impl;
};


#endif // OutputComponent_h
