/* Copyright (c) 2011  Eric Milles <eric.milles@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef DeviceInfo_h
#define DeviceInfo_h


#include <functional>
#include <set>
#include <string>
#include <utility>


class DeviceInfo
{
public:
	enum DeviceType
	{
		APX = 1
	};

	DeviceInfo(const DeviceType& type, const std::string& name,
		const std::pair<std::string,std::string>& addr, bool zeroConf);

	const DeviceType& type() const;
	const std::string& name() const;
	const std::pair<std::string,std::string>& addr() const;

	bool isZeroConf() const;

private:
	DeviceType _type;
	std::string _name;
	std::pair<std::string,std::string> _addr;
	bool _zeroConf;
};


bool operator ==(const DeviceInfo&, const DeviceInfo&);


class DeviceInfoNameLess
:
	public std::binary_function<DeviceInfo,DeviceInfo,bool>
{
public:
	result_type operator ()(const first_argument_type& lhs,
		const second_argument_type& rhs) const
	{
		return (lhs.name() < rhs.name());
	}
};


typedef std::set<const DeviceInfo,DeviceInfoNameLess> DeviceInfoSet;


#endif // DeviceInfo_h
