/* Copyright (c) 2011  Eric Milles <eric.milles@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef Options_h
#define Options_h


#include "DeviceInfo.h"
#include <map>
#include <set>
#include <string>
#include <utility>
#include <Poco/SharedPtr.h>


typedef Poco::SharedPtr<class Options> OptionsPtr;


class Options
{
	friend bool operator ==(const Options&, const Options&);

public:
	static OptionsPtr getOptions();
	static void setOptions(OptionsPtr);

	bool getBalanceAndVolumeControl() const;
	void setBalanceAndVolumeControl(bool);
	bool getPlayerControl() const;
	void setPlayerControl(bool);
	bool getResetOnPause() const;
	void setResetOnPause(bool);

	const DeviceInfoSet& devices() const;
	DeviceInfoSet& devices();

	bool isActivated(const std::string& deviceName) const;
	void setActivated(const std::string& deviceName, bool);
	const std::string& getPassword(const std::string& deviceName) const;
	bool getRememberPassword(const std::string& deviceName) const;
	void setPassword(const std::string& deviceName, const std::string& password,
		bool rememberPassword);
	void clearPassword(const std::string& deviceName);

private:
	static OptionsPtr _options;

	bool _balanceAndVolumeControl;
	bool _playerControl;
	bool _resetOnPause;

	DeviceInfoSet _devices;
	std::set<std::string> _activatedDevices;
	std::map<std::string,std::pair<std::string,bool>> _devicePasswords;
};


bool operator ==(const Options&, const Options&);


#endif // Options_h
