/* Copyright (c) 2011  Eric Milles <eric.milles@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "Debugger.h"
#include <windows.h>
#include <Poco/Debugger.h>
#include <cassert>


bool Debugger::isAvailable()
{
	return Poco::Debugger::isAvailable();
}


void Debugger::print(const std::string& msg)
{
	if (isAvailable())
	{
		Poco::Debugger::message(msg);
	}
}


#if defined(_MSC_VER) && defined(_WIN32)
#pragma pack(push,8)
struct ThreadNameInfo
{
	ThreadNameInfo(const std::string& name)
	:
		_type(0x1000),
		_name(name.c_str()),
		_id(GetCurrentThreadId()),
		_flags(0)
	{
	}

	DWORD _type;
	LPCSTR _name;
	DWORD _id;
	DWORD _flags;
};
#pragma pack(pop)
#endif


void Debugger::setThreadName(const std::string& name)
{
	if (isAvailable())
	{
#if defined(_MSC_VER) && defined(_WIN32)
		Sleep(10);
		ThreadNameInfo info(name);
		DWORD argCount = sizeof(info) / sizeof(ULONG_PTR);
		assert(argCount == 4); // type, name, id and flags
		__try
		{
			RaiseException(0x406D1388, 0, argCount, (ULONG_PTR*) &info);
		}
		__except(EXCEPTION_EXECUTE_HANDLER)
		{
		}
#endif
	}
}
