/* Copyright (c) 2011  Eric Milles <eric.milles@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef DeviceManager_h
#define DeviceManager_h


#include "Device.h"
#include "DeviceInfo.h"
#include "DeviceNotification.h"
#include "OutputFormat.h"
#include "OutputSink.h"
#include "Player.h"
#include "Uncopyable.h"
#include <map>
#include <memory>
#include <string>
#include <Poco/Mutex.h>
#include <Poco/Observer.h>


class DeviceManager
:
	private Uncopyable
{
	friend class DeviceWriter;

public:
	DeviceManager(const double&, const double&, Player&);
	~DeviceManager();

	bool isAnyDeviceOpen() const;
	void openDevices();
	void closeDevices();

	void setBalance();
	void setVolume();

private:
	Device* createDevice(const DeviceInfo&);
	void destroyDevice(const DeviceInfo&);

	bool isDeviceOpen(const DeviceInfo&) const;
	void openDevice(const DeviceInfo&);
	void closeDevice(const DeviceInfo&);

	void setDeviceBalance(const DeviceInfo&, double);
	void setDeviceVolume(const DeviceInfo&, double);

	void deviceChanged(DeviceNotification*);

public:
	bool isValidRemoteControlId(unsigned int) const;

	std::auto_ptr<OutputSink> createOutputSinkForDevices(const OutputFormat&);

private:
	const double& _balance;
	const double& _volume;

	typedef std::map<std::string,Device*> DeviceMap;
	DeviceMap _devices;

	bool _writer;

	Player& _player;
	Poco::Observer<DeviceManager,DeviceNotification> _observer;

	typedef Poco::FastMutex Mutex;
	mutable Mutex _mutex;
};


#endif // DeviceManager_h
