/* Copyright (c) 2011  Eric Milles <eric.milles@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef OutputThread_h
#define OutputThread_h


#include "OutputSink.h"
#include "Platform.h"
#include "Uncopyable.h"
#include <memory>
#include <vector>
#include <Poco/Mutex.h>
#include <Poco/Runnable.h>
#include <Poco/Thread.h>


class OutputThread
:
	public OutputSink,
	public Poco::Runnable,
	private Uncopyable
{
public:
	OutputThread(std::auto_ptr<OutputSink>);
	~OutputThread();

	std::time_t latency(const OutputFormat&) const;
	std::size_t buffered() const;
	std::size_t canWrite() const;

	void write(const void*, std::size_t);
	void flush();
	void reset();

	void run();

private:
	std::vector<byte_t> _buffer;
	std::size_t _bufferAvailability;
	std::size_t _bufferReadIndex;
	std::size_t _bufferWriteIndex;

	const std::auto_ptr<OutputSink> _outputSink;

	volatile bool _flushThread;
	volatile bool _stopThread;
	Poco::Thread _thread;

	typedef Poco::FastMutex Mutex;
	mutable Mutex _mutex;
};


#endif // OutputThread_h
