/* Copyright (c) 2011  Eric Milles <eric.milles@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "Platform.h"
#include <cassert>
#include <vector>
#include <Poco/UnicodeConverter.h>


using Poco::UnicodeConverter;


void Platform::Charset::fromUTF8(const std::string& src, std::string& dst)
{
	std::wstring uString;

	// transcode string from UTF-8 to UTF-16 representation
	UnicodeConverter::toUTF16(src, uString);

	// determine length of buffer necessary to hold ACP version of string
	const int length = WideCharToMultiByte(
		CP_ACP, 0, uString.c_str(), -1, NULL, 0, NULL, NULL);
	if (length <= 0) {
		throw std::runtime_error(
			"WideCharToMultiByte failed with error code " + GetLastError());
	}

	// allocate space for ACP version of string
	std::vector<char> aString(length);

	// transcode string from UTF-16 to ACP representation
	const int result = WideCharToMultiByte(
		CP_ACP, 0, uString.c_str(), -1, &aString[0], length, NULL, NULL);
	if (result <= 0) {
		throw std::runtime_error(
			"WideCharToMultiByte failed with error code " + GetLastError());
	}

	dst = &aString[0];
}


void Platform::Charset::fromUTF8(const std::string& src, std::wstring& dst)
{
	// transcode string from UTF-8 to UTF-16 representation
	UnicodeConverter::toUTF16(src, dst);
}


void Platform::Charset::toUTF8(const std::string& src, std::string& dst)
{
	// determine length of buffer necessary to hold UTF-16 version of string
	const int length = MultiByteToWideChar(CP_ACP, 0, src.c_str(), -1, NULL, 0);
	if (length <= 0) {
		throw std::runtime_error(
			"MultiByteToWideChar failed with error code " + GetLastError());
	}

	// allocate space for UTF-16 version of string
	std::vector<wchar_t> uString(length);

	// transcode string from ACP to UTF-16 representation
	const int result = MultiByteToWideChar(
		CP_ACP, 0, src.c_str(), -1, &uString[0], length);
	if (result <= 0) {
		throw std::runtime_error(
			"MultiByteToWideChar failed with error code " + GetLastError());
	}

	// transcode string from UTF-16 to UTF-8 representation
	UnicodeConverter::toUTF8(&uString[0], dst);
}


void Platform::Charset::toUTF8(const std::wstring& src, std::string& dst)
{
	// transcode string from UTF-16 to UTF-8 representation
	UnicodeConverter::toUTF8(src, dst);
}


//------------------------------------------------------------------------------


void Platform::centerWindow(HWND window)
{
	HWND parent = GetParent(window);
	assert(parent != NULL);
	RECT parentRect;
	GetWindowRect(parent, &parentRect);
	RECT windowRect;
	GetWindowRect(window, &windowRect);
	POINT centerPoint;
	centerPoint.x = parentRect.left + ((parentRect.right - parentRect.left) / 2);
	centerPoint.y = parentRect.top + ((parentRect.bottom - parentRect.top) / 2);
	POINT originPoint;
	originPoint.x = centerPoint.x - ((windowRect.right - windowRect.left) / 2);
	originPoint.y = centerPoint.y - ((windowRect.bottom - windowRect.top) / 2);
	SetWindowPos(window, HWND_TOP, originPoint.x, originPoint.y, 0, 0, SWP_NOSIZE);
}


void Platform::enableWindow(HWND window, const bool state)
{
	EnableWindow(window, state);
}
