/* Copyright (c) 2011  Eric Milles <eric.milles@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef Output_h
#define Output_h


#include "OptionsSubmenu.h"
#include "OutputComponent.h"
#include "Uncopyable.h"
#include <cstddef>
#include <ctime>
#include <memory>
#include <string>
#include <utility>
#include <windows.h>


class Output
:
	private Uncopyable
{
public:
	static void init();
	static void quit();

	static void about(HWND);
	static void options(HWND);

	static int open(int, int, int, int, int);
	static void close();
	static void reset(int);

	static int write(char*, int);

	static int canWrite();
	static int isPlaying();
	static int getOutputTime();
	static int getWrittenTime();

	static int setPaused(int);
	static void setBalance(int);
	static void setVolume(int);

	static void bytesOutput(std::size_t);

private:
	static INT_PTR CALLBACK optionsDialogProc(HWND, UINT, WPARAM, LPARAM);

	typedef std::pair<int,double> Progress;
	static void updateProgress(Progress&, std::size_t);

	static Output _this;
	static std::string _iniFilePath;
	static std::auto_ptr<OptionsSubmenu> _optionsSubmenu;
	static std::auto_ptr<OutputComponent> _outputComponent;
	static Progress _bufferProgress;
	static Progress _outputProgress;
	static Progress::second_type _msecDivisor;
};


#endif // Output_h
