/* Copyright (c) 2011  Eric Milles <eric.milles@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "Platform.h"
#include "WinampPlayer.h"
#include <cassert>
#include <wa_ipc.h>


WinampPlayer::WinampPlayer(HWND& playerWindow)
:
	_playerWindow(playerWindow)
{
}


HWND WinampPlayer::window() const
{
	return _playerWindow;
}


void WinampPlayer::play()
{
	// send play message to player window
	LRESULT result = SendMessage(_playerWindow, WM_COMMAND, 40045, 0);
	assert(result == 0);
}


void WinampPlayer::pause()
{
	// send pause message to player window
	LRESULT result = SendMessage(_playerWindow, WM_COMMAND, 40046, 0);
	assert(result == 0);
}


void WinampPlayer::stop()
{
	// send stop message to player window
	LRESULT result = SendMessage(_playerWindow, WM_COMMAND, 40047, 0);
	assert(result == 0);
}


void WinampPlayer::skipForward()
{
	// send skip forward message to player window
	LRESULT result = SendMessage(_playerWindow, WM_COMMAND, 40048, 0);
	assert(result == 0);
}


void WinampPlayer::skipBackward()
{
	// send skip backward message to player window
	LRESULT result = SendMessage(_playerWindow, WM_COMMAND, 40044, 0);
	assert(result == 0);
}


void WinampPlayer::seekForward()
{
	// send seek forward 5 seconds message to player window
	LRESULT result = SendMessage(_playerWindow, WM_COMMAND, 40060, 0);
	assert(result == 0);
}


void WinampPlayer::seekBackward()
{
	// send seek backward 5 seconds message to player window
	LRESULT result = SendMessage(_playerWindow, WM_COMMAND, 40061, 0);
	assert(result == 0);
}


void WinampPlayer::increaseVolume()
{
	// send volume up message to player window (twice)
	LRESULT result = SendMessage(_playerWindow, WM_COMMAND, 40058, 0);
	assert(result == 0);
	result = SendMessage(_playerWindow, WM_COMMAND, 40058, 0);
	assert(result == 0);
}


void WinampPlayer::decreaseVolume()
{
	// send volume down message to player window (twice)
	LRESULT result = SendMessage(_playerWindow, WM_COMMAND, 40059, 0);
	assert(result == 0);
	result = SendMessage(_playerWindow, WM_COMMAND, 40059, 0);
	assert(result == 0);
}


void WinampPlayer::mute()
{
	static uint8_t previousVolume = 0;

	// send get volume message to player window
	LRESULT result = SendMessage(_playerWindow, WM_WA_IPC,
		static_cast<WPARAM>(-666), IPC_SETVOLUME);
	assert(result >= 0 && result <= 255);

	if (result == 0 && previousVolume > 0)
	{
		// send set volume message to player window
		result = SendMessage(_playerWindow, WM_WA_IPC,
			static_cast<WPARAM>(previousVolume), IPC_SETVOLUME);
		assert(result == 0);

		previousVolume = 0;
	}
	else
	{
		// send set volume message to player window
		result = SendMessage(_playerWindow, WM_WA_IPC, 0, IPC_SETVOLUME);
		assert(result == 0);
	}
}


void WinampPlayer::shuffle()
{
	// send get shuffle state message to player window
	LRESULT result = SendMessage(_playerWindow, WM_WA_IPC, 0, IPC_GET_SHUFFLE);
	// send set shuffle state message to player window
	result = SendMessage(_playerWindow, WM_WA_IPC, !result, IPC_SET_SHUFFLE);
	assert(result == 0);
}
